@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -83,8 +83,6 @@ QT_BEGIN_NAMESPACE
     \ingroup multimedia_audio_qml
     \inqmlmodule QtMultimedia
 
-    SoundEffect is part of the \b{QtMultimedia 5.0} module.
-
     This type allows you to play uncompressed audio files (typically WAV files) in
     a generally lower latency way, and is suitable for "feedback" type sounds in
     response to user actions (e.g. virtual keyboard sounds, positive or negative
@@ -275,11 +273,7 @@ qreal QSoundEffect::volume() const
  */
 void QSoundEffect::setVolume(qreal volume)
 {
-    if (volume < qreal(0.0) || volume > qreal(1.0)) {
-        qWarning("SoundEffect: volume should be between 0.0 and 1.0");
-        return;
-    }
-
+    volume = qBound(qreal(0.0), volume, qreal(1.0));
     if (qFuzzyCompare(d->volume(), volume))
         return;
 
