@@ -42,6 +42,7 @@
 #include <qmediaplaylistcontrol_p.h>
 #include <qmediaplaylistsourcecontrol_p.h>
 #include <qmedianetworkaccesscontrol.h>
+#include <qaudiorolecontrol.h>
 
 #include <QtCore/qcoreevent.h>
 #include <QtCore/qmetaobject.h>
@@ -93,7 +94,6 @@ static void qRegisterMediaPlayerMetaTypes()
 
 Q_CONSTRUCTOR_FUNCTION(qRegisterMediaPlayerMetaTypes)
 
-
 #define MAX_NESTED_PLAYLISTS 16
 
 class QMediaPlayerPrivate : public QMediaObjectPrivate
@@ -104,37 +104,40 @@ public:
     QMediaPlayerPrivate()
         : provider(0)
         , control(0)
+        , audioRoleControl(0)
+        , playlist(0)
+        , networkAccessControl(0)
         , state(QMediaPlayer::StoppedState)
         , status(QMediaPlayer::UnknownMediaStatus)
         , error(QMediaPlayer::NoError)
         , ignoreNextStatusChange(-1)
-        , playlist(0)
-        , networkAccessControl(0)
-        , hasStreamPlaybackFeature(false)
         , nestedPlaylists(0)
+        , hasStreamPlaybackFeature(false)
     {}
 
     QMediaServiceProvider *provider;
     QMediaPlayerControl* control;
-    QMediaPlayer::State state;
-    QMediaPlayer::MediaStatus status;
-    QMediaPlayer::Error error;
+    QAudioRoleControl *audioRoleControl;
     QString errorString;
-    int ignoreNextStatusChange;
 
     QPointer<QObject> videoOutput;
     QMediaPlaylist *playlist;
     QMediaNetworkAccessControl *networkAccessControl;
     QVideoSurfaceOutput surfaceOutput;
-    bool hasStreamPlaybackFeature;
     QMediaContent qrcMedia;
     QScopedPointer<QFile> qrcFile;
 
     QMediaContent rootMedia;
     QMediaContent pendingPlaylist;
+    QMediaPlayer::State state;
+    QMediaPlayer::MediaStatus status;
+    QMediaPlayer::Error error;
+    int ignoreNextStatusChange;
+    int nestedPlaylists;
+    bool hasStreamPlaybackFeature;
+
     QMediaPlaylist *parentPlaylist(QMediaPlaylist *pls);
     bool isInChain(QUrl url);
-    int nestedPlaylists;
 
     void setMedia(const QMediaContent &media, QIODevice *stream = 0);
 
@@ -598,6 +601,12 @@ QMediaPlayer::QMediaPlayer(QObject *parent, QMediaPlayer::Flags flags):
                 addPropertyWatch("bufferStatus");
 
             d->hasStreamPlaybackFeature = d->provider->supportedFeatures(d->service).testFlag(QMediaServiceProviderHint::StreamPlayback);
+
+            d->audioRoleControl = qobject_cast<QAudioRoleControl*>(d->service->requestControl(QAudioRoleControl_iid));
+            if (d->audioRoleControl) {
+                connect(d->audioRoleControl, &QAudioRoleControl::audioRoleChanged,
+                        this, &QMediaPlayer::audioRoleChanged);
+            }
         }
         if (d->networkAccessControl != 0) {
             connect(d->networkAccessControl, SIGNAL(configurationChanged(QNetworkConfiguration)),
@@ -620,6 +629,8 @@ QMediaPlayer::~QMediaPlayer()
     if (d->service) {
         if (d->control)
             d->service->releaseControl(d->control);
+        if (d->audioRoleControl)
+            d->service->releaseControl(d->audioRoleControl);
 
         d->provider->releaseService(d->service);
     }
@@ -842,17 +853,19 @@ void QMediaPlayer::play()
     }
 
     //if playlist control is available, the service should advance itself
-    if (d->rootMedia.playlist() && d->rootMedia.playlist()->currentIndex() == -1 && !d->rootMedia.playlist()->isEmpty()) {
-
+    if (d->rootMedia.playlist() && !d->rootMedia.playlist()->isEmpty()) {
         // switch to playing state
         if (d->state != QMediaPlayer::PlayingState)
             d->_q_stateChanged(QMediaPlayer::PlayingState);
 
-        if (d->playlist != d->rootMedia.playlist())
-            d->setPlaylist(d->rootMedia.playlist());
-        Q_ASSERT(d->playlist == d->rootMedia.playlist());
-        emit currentMediaChanged(d->rootMedia);
-        d->playlist->setCurrentIndex(0);
+        if (d->rootMedia.playlist()->currentIndex() == -1) {
+            if (d->playlist != d->rootMedia.playlist())
+                d->setPlaylist(d->rootMedia.playlist());
+            Q_ASSERT(d->playlist == d->rootMedia.playlist());
+
+            emit currentMediaChanged(d->rootMedia);
+            d->playlist->setCurrentIndex(0);
+        }
     }
 
     // Reset error conditions
@@ -943,8 +956,8 @@ void QMediaPlayer::setPlaybackRate(qreal rate)
     Sets the current \a media source.
 
     If a \a stream is supplied; media data will be read from it instead of resolving the media
-    source.  In this case the media source may still be used to resolve additional information
-    about the media such as mime type.
+    source. In this case the media source may still be used to resolve additional information
+    about the media such as mime type. The \a stream must be open and readable.
 
     Setting the media to a null QMediaContent will cause the player to discard all
     information relating to the current media source and to cease all I/O operations related
@@ -1113,6 +1126,41 @@ QMultimedia::AvailabilityStatus QMediaPlayer::availability() const
     return QMediaObject::availability();
 }
 
+QAudio::Role QMediaPlayer::audioRole() const
+{
+    Q_D(const QMediaPlayer);
+
+    if (d->audioRoleControl != NULL)
+        return d->audioRoleControl->audioRole();
+
+    return QAudio::UnknownRole;
+}
+
+void QMediaPlayer::setAudioRole(QAudio::Role audioRole)
+{
+    Q_D(QMediaPlayer);
+
+    if (d->audioRoleControl)
+        d->audioRoleControl->setAudioRole(audioRole);
+}
+
+/*!
+    Returns a list of supported audio roles.
+
+    If setting the audio role is not supported, an empty list is returned.
+
+    \since 5.6
+    \sa audioRole
+*/
+QList<QAudio::Role> QMediaPlayer::supportedAudioRoles() const
+{
+    Q_D(const QMediaPlayer);
+
+    if (d->audioRoleControl)
+        return d->audioRoleControl->supportedAudioRoles();
+
+    return QList<QAudio::Role>();
+}
 
 // Enums
 /*!
@@ -1213,6 +1261,14 @@ QMultimedia::AvailabilityStatus QMediaPlayer::availability() const
     Signals the \a seekable status of the player object has changed.
 */
 
+/*!
+    \fn void QMediaPlayer::audioRoleChanged(QAudio::Role role)
+
+    Signals that the audio \a role of the media player has changed.
+
+    \since 5.6
+*/
+
 // Properties
 /*!
     \property QMediaPlayer::state
@@ -1380,6 +1436,19 @@ QMultimedia::AvailabilityStatus QMediaPlayer::availability() const
     while fast forwarding or rewinding.
 */
 
+/*!
+    \property QMediaPlayer::audioRole
+    \brief the role of the audio stream played by the media player.
+
+    It can be set to specify the type of audio being played, allowing the system to make
+    appropriate decisions when it comes to volume, routing or post-processing.
+
+    The audio role must be set before calling setMedia().
+
+    \since 5.6
+    \sa supportedAudioRoles()
+*/
+
 /*!
     \fn void QMediaPlayer::durationChanged(qint64 duration)
 
