@@ -53,11 +53,11 @@ public:
         if (--mControl->mActiveStateChangeNotifiers)
             return;
 
-        if (mPreviousState != mControl->state())
-            Q_EMIT mControl->stateChanged(mControl->state());
-
         if (mPreviousMediaStatus != mControl->mediaStatus())
             Q_EMIT mControl->mediaStatusChanged(mControl->mediaStatus());
+
+        if (mPreviousState != mControl->state())
+            Q_EMIT mControl->stateChanged(mControl->state());
     }
 
 private:
@@ -345,7 +345,7 @@ void QAndroidMediaPlayerControl::setMedia(const QMediaContent &mediaContent,
     mReloadingMedia = false;
 }
 
-void QAndroidMediaPlayerControl::setVideoOutput(QObject *videoOutput)
+void QAndroidMediaPlayerControl::setVideoOutput(QAndroidVideoOutput *videoOutput)
 {
     if (mVideoOutput) {
         mMediaPlayer->setDisplay(0);
@@ -353,7 +353,7 @@ void QAndroidMediaPlayerControl::setVideoOutput(QObject *videoOutput)
         mVideoOutput->reset();
     }
 
-    mVideoOutput = qobject_cast<QAndroidVideoOutput *>(videoOutput);
+    mVideoOutput = videoOutput;
 
     if (!mVideoOutput)
         return;
@@ -563,6 +563,7 @@ void QAndroidMediaPlayerControl::onStateChanged(qint32 state)
         } else {
             onBufferingChanged(100);
         }
+        Q_EMIT metaDataUpdated();
         setAudioAvailable(true);
         flushPendingStates();
         break;
