@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -62,13 +54,11 @@ QT_BEGIN_NAMESPACE
 
 QGstreamerPlayerControl::QGstreamerPlayerControl(QGstreamerPlayerSession *session, QObject *parent)
     : QMediaPlayerControl(parent)
-    , m_ownStream(false)
     , m_session(session)
     , m_userRequestedState(QMediaPlayer::StoppedState)
     , m_currentState(QMediaPlayer::StoppedState)
     , m_mediaStatus(QMediaPlayer::NoMedia)
     , m_bufferProgress(-1)
-    , m_seekToStartPending(false)
     , m_pendingSeekPosition(-1)
     , m_setMediaPending(false)
     , m_stream(0)
@@ -77,7 +67,7 @@ QGstreamerPlayerControl::QGstreamerPlayerControl(QGstreamerPlayerSession *sessio
     Q_ASSERT(m_resources);
 
     connect(m_session, SIGNAL(positionChanged(qint64)),
-            this, SLOT(updatePosition(qint64)));
+            this, SIGNAL(positionChanged(qint64)));
     connect(m_session, SIGNAL(durationChanged(qint64)),
             this, SIGNAL(durationChanged(qint64)));
     connect(m_session, SIGNAL(mutedStateChanged(bool)),
@@ -102,8 +92,6 @@ QGstreamerPlayerControl::QGstreamerPlayerControl(QGstreamerPlayerSession *sessio
             this, SLOT(handleInvalidMedia()));
     connect(m_session, SIGNAL(playbackRateChanged(qreal)),
             this, SIGNAL(playbackRateChanged(qreal)));
-    connect(m_session, SIGNAL(seekableChanged(bool)),
-            this, SLOT(applyPendingSeek(bool)));
 
     connect(m_resources, SIGNAL(resourcesGranted()), SLOT(handleResourcesGranted()));
     //denied signal should be queued to have correct state update process,
@@ -125,7 +113,7 @@ QMediaPlayerResourceSetInterface* QGstreamerPlayerControl::resources() const
 
 qint64 QGstreamerPlayerControl::position() const
 {
-    return m_seekToStartPending ? 0 : m_session->position();
+    return m_pendingSeekPosition != -1 ? m_pendingSeekPosition : m_session->position();
 }
 
 qint64 QGstreamerPlayerControl::duration() const
@@ -191,15 +179,21 @@ void QGstreamerPlayerControl::setPosition(qint64 pos)
 
     if (m_mediaStatus == QMediaPlayer::EndOfMedia) {
         m_mediaStatus = QMediaPlayer::LoadedMedia;
-        m_seekToStartPending = true;
     }
 
-    if (m_session->isSeekable() && m_session->seek(pos)) {
-        m_seekToStartPending = false;
-    } else {
+    if (m_currentState == QMediaPlayer::StoppedState) {
         m_pendingSeekPosition = pos;
-        //don't display the first video frame since it's not what user requested.
-        m_session->showPrerollFrames(false);
+        emit positionChanged(m_pendingSeekPosition);
+    } else if (m_session->isSeekable()) {
+        m_session->showPrerollFrames(true);
+        m_session->seek(pos);
+        m_pendingSeekPosition = -1;
+    } else if (m_session->state() == QMediaPlayer::StoppedState) {
+        m_pendingSeekPosition = pos;
+        emit positionChanged(m_pendingSeekPosition);
+    } else if (m_pendingSeekPosition != -1) {
+        m_pendingSeekPosition = -1;
+        emit positionChanged(m_pendingSeekPosition);
     }
 
     popAndNotifyState();
@@ -247,26 +241,30 @@ void QGstreamerPlayerControl::playOrPause(QMediaPlayer::State newState)
     }
 #endif
 
+    if (m_mediaStatus == QMediaPlayer::EndOfMedia && m_pendingSeekPosition == -1) {
+        m_pendingSeekPosition = 0;
+    }
+
     if (!m_resources->isGranted())
         m_resources->acquire();
 
     if (m_resources->isGranted()) {
-        if (m_seekToStartPending) {
+        // show prerolled frame if switching from stopped state
+        if (m_pendingSeekPosition == -1) {
+            m_session->showPrerollFrames(true);
+        } else if (m_session->state() == QMediaPlayer::StoppedState) {
+            // Don't evaluate the next two conditions.
+        } else if (m_session->isSeekable()) {
             m_session->pause();
-            if (!m_session->seek(0)) {
-                m_bufferProgress = -1;
-                m_session->stop();
-                m_mediaStatus = QMediaPlayer::LoadingMedia;
-            }
-            m_seekToStartPending = false;
+            m_session->showPrerollFrames(true);
+            m_session->seek(m_pendingSeekPosition);
+            m_pendingSeekPosition = -1;
+        } else {
+            m_pendingSeekPosition = -1;
         }
 
         bool ok = false;
 
-        // show prerolled frame if switching from stopped state
-        if (newState != QMediaPlayer::StoppedState && m_currentState == QMediaPlayer::StoppedState && m_pendingSeekPosition == -1)
-            m_session->showPrerollFrames(true);
-
         //To prevent displaying the first video frame when playback is resumed
         //the pipeline is paused instead of playing, seeked to requested position,
         //and after seeking is finished (position updated) playback is restarted
@@ -313,7 +311,7 @@ void QGstreamerPlayerControl::stop()
             m_session->pause();
 
         if (m_mediaStatus != QMediaPlayer::EndOfMedia) {
-            m_seekToStartPending = true;
+            m_pendingSeekPosition = 0;
             emit positionChanged(position());
         }
     }
@@ -351,7 +349,7 @@ void QGstreamerPlayerControl::setMedia(const QMediaContent &content, QIODevice *
 
     m_currentState = QMediaPlayer::StoppedState;
     QMediaContent oldMedia = m_currentResource;
-    m_pendingSeekPosition = -1;
+    m_pendingSeekPosition = 0;
     m_session->showPrerollFrames(false); // do not show prerolled frames until pause() or play() explicitly called
     m_setMediaPending = false;
 
@@ -371,34 +369,8 @@ void QGstreamerPlayerControl::setMedia(const QMediaContent &content, QIODevice *
         emit bufferStatusChanged(0);
     }
 
-    if (m_stream) {
-        if (m_ownStream)
-            delete m_stream;
-        m_stream = 0;
-        m_ownStream = false;
-    }
-
-    // If the canonical URL refers to a Qt resource, open with QFile and use
-    // the stream playback capability to play.
-    if (stream == 0 && content.canonicalUrl().scheme() == QLatin1String("qrc")) {
-        stream = new QFile(QLatin1Char(':') + content.canonicalUrl().path(), this);
-        if (!stream->open(QIODevice::ReadOnly)) {
-            delete stream;
-            m_mediaStatus = QMediaPlayer::InvalidMedia;
-            m_currentResource = content;
-            emit mediaChanged(m_currentResource);
-            emit error(QMediaPlayer::FormatError, tr("Attempting to play invalid Qt resource"));
-            if (m_currentState != QMediaPlayer::PlayingState)
-                m_resources->release();
-            popAndNotifyState();
-            return;
-        }
-        m_ownStream = true;
-    }
-
     m_currentResource = content;
     m_stream = stream;
-    m_seekToStartPending = false;
 
     QNetworkRequest request;
 
@@ -427,7 +399,6 @@ void QGstreamerPlayerControl::setMedia(const QMediaContent &content, QIODevice *
         m_session->loadFromUri(request);
 #endif
 
-
 #if defined(HAVE_GST_APPSRC)
     if (!request.url().isEmpty() || userStreamValid) {
 #else
@@ -470,8 +441,21 @@ void QGstreamerPlayerControl::updateSessionState(QMediaPlayer::State state)
 {
     pushState();
 
-    if (state == QMediaPlayer::StoppedState)
+    if (state == QMediaPlayer::StoppedState) {
+        m_session->showPrerollFrames(false);
         m_currentState = QMediaPlayer::StoppedState;
+    }
+
+    if (state == QMediaPlayer::PausedState && m_currentState != QMediaPlayer::StoppedState) {
+        if (m_pendingSeekPosition != -1 && m_session->isSeekable()) {
+            m_session->showPrerollFrames(true);
+            m_session->seek(m_pendingSeekPosition);
+        }
+        m_pendingSeekPosition = -1;
+
+        if (m_currentState == QMediaPlayer::PlayingState)
+            m_session->play();
+    }
 
     updateMediaStatus();
 
@@ -520,7 +504,6 @@ void QGstreamerPlayerControl::processEOS()
     m_mediaStatus = QMediaPlayer::EndOfMedia;
     emit positionChanged(position());
     m_session->endOfMediaReset();
-    m_setMediaPending = true;
 
     if (m_currentState != QMediaPlayer::StoppedState) {
         m_currentState = QMediaPlayer::StoppedState;
@@ -557,17 +540,12 @@ void QGstreamerPlayerControl::setBufferProgress(int progress)
     emit bufferStatusChanged(m_bufferProgress);
 }
 
-void QGstreamerPlayerControl::applyPendingSeek(bool isSeekable)
-{
-    if (isSeekable && m_pendingSeekPosition != -1)
-        setPosition(m_pendingSeekPosition);
-}
-
 void QGstreamerPlayerControl::handleInvalidMedia()
 {
     pushState();
     m_mediaStatus = QMediaPlayer::InvalidMedia;
     m_currentState = QMediaPlayer::StoppedState;
+    m_setMediaPending = true;
     popAndNotifyState();
 }
 
@@ -628,40 +606,20 @@ void QGstreamerPlayerControl::popAndNotifyState()
     QMediaPlayer::MediaStatus oldMediaStatus = m_mediaStatusStack.pop();
 
     if (m_stateStack.isEmpty()) {
-        if (m_currentState != oldState) {
-#ifdef DEBUG_PLAYBIN
-            qDebug() << "State changed:" << m_state;
-#endif
-            emit stateChanged(m_currentState);
-        }
-
         if (m_mediaStatus != oldMediaStatus) {
 #ifdef DEBUG_PLAYBIN
             qDebug() << "Media status changed:" << m_mediaStatus;
 #endif
             emit mediaStatusChanged(m_mediaStatus);
         }
-    }
-}
 
-void QGstreamerPlayerControl::updatePosition(qint64 pos)
-{
+        if (m_currentState != oldState) {
 #ifdef DEBUG_PLAYBIN
-    qDebug() << Q_FUNC_INFO << pos/1000.0 << "pending:" << m_pendingSeekPosition/1000.0;
+            qDebug() << "State changed:" << m_currentState;
 #endif
-
-    if (m_pendingSeekPosition != -1) {
-        //seek request is complete, it's safe to resume playback
-        //with prerolled frame displayed
-        m_pendingSeekPosition = -1;
-        if (m_currentState != QMediaPlayer::StoppedState)
-            m_session->showPrerollFrames(true);
-        if (m_currentState == QMediaPlayer::PlayingState) {
-            m_session->play();
+            emit stateChanged(m_currentState);
         }
     }
-
-    emit positionChanged(pos);
 }
 
 QT_END_NAMESPACE
